@@ -17,21 +17,59 @@
    along with GNU Radius; if not, write to the Free Software Foundation, 
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define SQLT_NONE     0
-#define SQLT_MYSQL    1
-#define SQLT_POSTGRES 2
-#define SQLT_ODBC     3
-#define SQLT_MAX      4
+#ifndef _gnu_radsql_h
+#define _gnu_radsql_h
+
+#ifdef HAVE_LIBLTDL
+# define DECL_SQL_DISPATCH_TAB(mod) \
+  SQL_DISPATCH_TAB RDL_EXPORT(mod,dispatch_tab)
+#else
+# define __s_cat2__(a,b) a ## b 
+# define DECL_SQL_DISPATCH_TAB(mod) \
+  SQL_DISPATCH_TAB __s_cat2__(mod,_dispatch_tab)
+#endif
 
 #ifdef USE_SQL
 
-#define SQL_AUTH     0
-#define SQL_ACCT     1
-#define SQL_NSERVICE 2
+# define SQL_AUTH     0
+# define SQL_ACCT     1
+# define SQL_NSERVICE 2
 
-#define SQL_CACHE_SIZE 16
+# define SQL_CACHE_SIZE 16
 typedef char **SQL_TUPLE;
 
+enum radius_sql_query {
+	auth_query,
+	group_query,
+	acct_start_query,
+        acct_stop_query,
+        acct_nasup_query,
+        acct_nasdown_query,
+        acct_keepalive_query,
+        check_attr_query,
+        reply_attr_query,
+	mlc_user_query,
+	mlc_realm_query,
+	mlc_stop_query,
+	auth_success_query,
+	auth_failure_query,
+	num_radius_sql_query
+};
+
+typedef struct {
+        int      interface;
+        char     *server;
+        int      port;
+        char     *login;
+        char     *password;
+        char     *acct_db;
+        char     *auth_db;
+	char     *query[num_radius_sql_query];
+        int      keepopen;
+        time_t   idle_timeout;
+        int      active[SQL_NSERVICE];
+} SQL_cfg;
+
 typedef struct {
 	char *query;
 	size_t ntuples;
@@ -40,6 +78,7 @@ typedef struct {
 } SQL_RESULT;
 
 struct sql_connection {
+	SQL_cfg *cfg;
 	int    interface;        /* One of SQLT_ values */
         int    type;             /* One of SQL_ values */
         int    connected;        /* Connected to the database? */
@@ -53,46 +92,8 @@ struct sql_connection {
 	size_t tail;
 };
 
-typedef struct {
-        int      interface;
-        char     *server;
-        int      port;
-        char     *login;
-        char     *password;
-        char     *acct_db;
-        char     *auth_db;
-        char     *auth_query;
-        char     *group_query;
-        char     *acct_start_query;
-        char     *acct_stop_query;
-        char     *acct_nasup_query;
-        char     *acct_nasdown_query;
-        char     *acct_keepalive_query;
-        char     *check_attr_query;
-        char     *reply_attr_query;
-        int      keepopen;
-        time_t   idle_timeout;
-        int      active[SQL_NSERVICE];
-} SQL_cfg;
-
-extern SQL_cfg sql_cfg;
-
-void radiusd_sql_shutdown();
-void radiusd_sql_clear_cache();
-
-int rad_sql_init();
-void rad_sql_acct(RADIUS_REQ *req);
-char *rad_sql_pass(RADIUS_REQ *req, char *data);
-void rad_sql_cleanup(int type, void *req);
-int rad_sql_checkgroup(RADIUS_REQ *req, char *groupname);
-int rad_sql_check_attr_query(RADIUS_REQ *req, VALUE_PAIR **check_pairs);
-int rad_sql_reply_attr_query(RADIUS_REQ *req, VALUE_PAIR **reply_pairs);
-
-#ifdef RADIUS_SERVER_GUILE
-SCM sql_exec_query(int type, char *query);
-#endif
-
 /* Dispatcher routines */
+void disp_init();
 int disp_sql_interface_index(char *name);
 int disp_sql_reconnect(int interface, int conn_type, struct sql_connection *conn);
 void disp_sql_disconnect(struct sql_connection *conn);
@@ -120,28 +121,5 @@ typedef struct {
 	int (*n_columns)(struct sql_connection *conn, void *data, size_t *np);
 } SQL_DISPATCH_TAB;
 
-#ifdef USE_SQL_MYSQL
-extern SQL_DISPATCH_TAB mysql_dispatch_tab[];
-#else
-#define mysql_dispatch_tab NULL
 #endif
-#ifdef USE_SQL_POSTGRES
-extern SQL_DISPATCH_TAB postgres_dispatch_tab[];
-#else
-#define postgres_dispatch_tab NULL
-#endif
-#ifdef USE_SQL_ODBC
-extern SQL_DISPATCH_TAB odbc_dispatch_tab[];
-#else
-#define odbc_dispatch_tab NULL
-#endif
-
-#else
-
-# define rad_sql_check_connect(a)
-# define rad_sql_setup NULL
-# define rad_sql_cleanup (void (*)(int, void *)) NULL
-# define rad_sql_shutdown()
-# define rad_sql_idle_check()
-
 #endif
