@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -30,6 +30,6 @@ int ts_parse(TIMESPAN **sp, char *str, char **endp);
 int ts_match(TIMESPAN *timespan, time_t *time_now, unsigned *rest);
 int ts_check(char *str, time_t *time, unsigned *rest, char **endp);
 
-#include <mem.h>
-#define ALLOC emalloc
-#define FREE efree
+#include <radius/mem.h>
+#define ALLOC grad_emalloc
+#define FREE grad_free
