@@ -1,5 +1,6 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2005,
+   2006 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
  
@@ -13,9 +14,10 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
   
-   You should have received a copy of the GNU General Public License
-   along with GNU Radius; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+   You should have received a copy of the GNU General Public
+   License along with GNU Radius; if not, write to the Free
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301 USA. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -401,7 +403,7 @@ _dict_attribute(struct parse_data *pd, int fc, char **fv, grad_locus_t *loc)
         if (nfields(fc, 4, 6, loc))
                 return 0;
 
-	debug(100,
+	GRAD_DEBUG(100,
 	      ("%s:%lu: ATTR_NAME=%s ATTR_VALUE=%s ATTR_TYPE=%s ATTR_VENDOR=%s ATTR_FLAGS=%s",
 	       loc->file, loc->line,
 	       ATTR_NAME,ATTR_VALUE,ATTR_TYPE,
@@ -696,7 +698,7 @@ _dict_begin(struct parse_data *pd, int fc, char **fv, grad_locus_t *loc)
 		return;
 	}
 	pd->begin_locus = *loc;
-	debug(1, ("%s:%lu: BEGIN VENDOR %s (%d)",
+	GRAD_DEBUG(1, ("%s:%lu: BEGIN VENDOR %s (%d)",
 		  loc->file, loc->line, fv[2], pd->vendor));
 }
 
@@ -707,7 +709,7 @@ _dict_end(struct parse_data *pd, int fc, char **fv, grad_locus_t *loc)
 		grad_log_loc(L_ERR, loc, _("unexpected END"));
 		pd->errcnt++;
 	}
-	debug(1, ("END VENDOR %d", pd->vendor));
+	GRAD_DEBUG(1, ("END VENDOR %d", pd->vendor));
 	pd->vendor = 0;
 }
 
@@ -826,8 +828,8 @@ parse_dict(char *name)
 	if (name[0] == '/')
 		path = grad_estrdup(name);
 	else
-		path = grad_mkfilename(radius_dir, name);
-	debug(1,("parsing %s", path));
+		path = grad_mkfilename(grad_config_dir, name);
+	GRAD_DEBUG(1,("parsing %s", path));
         rc = grad_read_raddb_file(path, 1, NULL, parse_dict_entry, &pd);
 	if (pd.vendor) {
 		grad_log_loc(L_ERR, &pd.begin_locus, _("BEGIN without END"));
