@@ -13,9 +13,10 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
   
-   You should have received a copy of the GNU General Public License
-   along with GNU Radius; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+   You should have received a copy of the GNU General Public
+   License along with GNU Radius; if not, write to the Free
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301 USA. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -28,6 +29,7 @@
 #include <errno.h>
 #include <string.h>
 #include <common.h>
+#include <radargp.h>
 
 #define SP(p) ((p)?(p):"")
 
@@ -134,7 +136,7 @@ _dolog(int level, char *file, size_t line, char *func_name, char *fmt, ...)
 }
 
 void
-_debug_print(char *file, size_t line, char *func_name, char *str)
+_grad_debug_print(char *file, size_t line, char *func_name, char *str)
 {
         _dolog(L_DEBUG, file, line, func_name, "%s", str);
         free(str);
@@ -142,7 +144,7 @@ _debug_print(char *file, size_t line, char *func_name, char *str)
 
 /*VARARGS*/
 char *
-_debug_format_string(char *fmt, ...)
+_grad_debug_format_string(char *fmt, ...)
 {
         va_list ap;
         char *str = NULL;
