@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2002,2003,2004,2006 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -13,9 +13,10 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
   
-   You should have received a copy of the GNU General Public License
-   along with GNU Radius; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+   You should have received a copy of the GNU General Public
+   License along with GNU Radius; if not, write to the Free
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301 USA. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -56,22 +57,29 @@ grad_server_send_reply(int fd, grad_request_t *radreq,
                 struct sockaddr_in *sin;
                 char buf[GRAD_MAX_LONGNAME];
 
-                debug(1, ("Sending %s of id %d to %lx (nas %s)",
-                          grad_request_code_to_name(reply_code), 
-                          radreq->id,
-			  (u_long)radreq->ipaddr,
-                          grad_nas_request_to_name(radreq, buf, sizeof buf)));
-                
                 sin = (struct sockaddr_in *) &saremote;
                 memset ((char *) sin, '\0', sizeof (saremote));
                 sin->sin_family = AF_INET;
                 sin->sin_addr.s_addr = htonl(radreq->ipaddr);
                 sin->sin_port = htons(radreq->udp_port);
-#ifndef DEBUG_ONLY
+#ifdef DEBUG_ONLY
+		grad_log(L_DEBUG,
+			 "DEBUG_ONLY version: not sending %s of id %d to %s (nas %s)",
+			 grad_request_code_to_name(reply_code), 
+			 radreq->id,
+			 grad_ip_iptostr(radreq->ipaddr, NULL),
+			 grad_nas_request_to_name(radreq, buf, sizeof buf));
+#else		
+                GRAD_DEBUG(1, ("Sending %s of id %d to %s (nas %s)",
+                          grad_request_code_to_name(reply_code), 
+                          radreq->id,
+			  grad_ip_iptostr(radreq->ipaddr, NULL),
+                          grad_nas_request_to_name(radreq, buf, sizeof buf)));
+                
                 sendto(fd, pdu, length, 0,
                        &saremote, sizeof(struct sockaddr_in));
 #endif
-                grad_free(pdu);
+		grad_free(pdu);
         }
 	return length;
 }
@@ -105,17 +113,22 @@ grad_server_send_challenge(int fd, grad_request_t *radreq,
                 struct sockaddr_in *sin;
                 char buf[GRAD_MAX_LONGNAME];
 
-                debug(1, ("Sending Challenge of id %d to %lx (nas %s)",
-                          radreq->id, (u_long)radreq->ipaddr,
-                          grad_nas_request_to_name(radreq, buf, sizeof buf)));
-        
                 sin = (struct sockaddr_in *) &saremote;
                 memset ((char *) sin, '\0', sizeof (saremote));
                 sin->sin_family = AF_INET;
                 sin->sin_addr.s_addr = htonl(radreq->ipaddr);
                 sin->sin_port = htons(radreq->udp_port);
 
-#ifndef DEBUG_ONLY
+#ifdef DEBUG_ONLY
+		grad_log(L_DEBUG,
+			 "DEBUG_ONLY version: not sending challenge of id %d to %s (nas %s)",
+			 radreq->id, grad_ip_iptostr(radreq->ipaddr, NULL),
+			 grad_nas_request_to_name(radreq, buf, sizeof buf)));
+#else
+                GRAD_DEBUG(1, ("Sending Challenge of id %d to %s (nas %s)",
+                          radreq->id, grad_ip_iptostr(radreq->ipaddr, NULL),
+                          grad_nas_request_to_name(radreq, buf, sizeof buf)));
+        
                 sendto(fd, pdu, length, 0,
                        &saremote, sizeof(struct sockaddr_in));
 #endif
