@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2001,2003 Free Software Foundation, Inc.
+   Copyright (C) 2001,2003,2004,2006 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
@@ -17,8 +17,6 @@
    along with GNU Radius; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define RADIUS_MODULE_PAM_C
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -61,8 +59,8 @@ rad_pam_conv(int num_msg, const struct pam_message **msg,
         int rc;
 
         if (!data) {
-                radlog(L_ERR,
-                       _("rad_pam_conv(): no application-specific data passed"));
+                grad_log(L_ERR,
+                         _("rad_pam_conv(): no application-specific data passed"));
                 return PAM_CONV_ERR;
         }
         
@@ -82,7 +80,7 @@ rad_pam_conv(int num_msg, const struct pam_message **msg,
                         break;
                 case PAM_ERROR_MSG:
                 case PAM_TEXT_INFO: 
-                        data->reply_msg = estrdup((char*)msg[count]->msg);
+                        data->reply_msg = grad_estrdup((char*)msg[count]->msg);
                         break;
                 default:
                         data->error++;
@@ -117,17 +115,17 @@ pam_pass(char *name, char *passwd, const char *pamauth, char **reply_msg)
 
         conv.appdata_ptr = &data;
 
-        debug(1,("username [%s], pamauth [%s]",  name, pamauth));
+        GRAD_DEBUG(1,("username [%s], pamauth [%s]",  name, pamauth));
 
         /* fake loop needed so we don't have to use gotos */
         for (;;) {
                 rc = pam_start(pamauth, name, &conv, &pamh);
-                debug(1, ("pam_start: %d", rc));
+                GRAD_DEBUG(1, ("pam_start: %d", rc));
                 if (rc != PAM_SUCCESS)
                         break;
 
                 rc = pam_authenticate(pamh, 0);
-                debug(1, ("pam_authenticate: %d", rc));
+                GRAD_DEBUG(1, ("pam_authenticate: %d", rc));
 
                 if (rc != PAM_SUCCESS) 
                         break;
@@ -135,7 +133,7 @@ pam_pass(char *name, char *passwd, const char *pamauth, char **reply_msg)
                 rc = pam_acct_mgmt(pamh, 0);
                 break;
         }
-        debug(1, ("pam_acct_mgmt: %d", rc));
+        GRAD_DEBUG(1, ("pam_acct_mgmt: %d", rc));
         pam_end(pamh, 0);
 
         *reply_msg = data.reply_msg;
