@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2003,2004,2006 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
@@ -99,17 +99,18 @@ pipe_write(int fd, void *ptr, size_t size, struct timeval *tv)
 			
 			rc = select(fd + 1, NULL, &wr_set, NULL, &tval);
 			if (rc == 0) {
-				debug(100, ("rc = 0"));
+				errno = ETIMEDOUT;
+				GRAD_DEBUG(100, ("rc = 0"));
 				break;
 			} else if (rc < 0) {
 				if (errno == EINTR) 
 					continue;
-				debug(100, ("rc = %d, errno = %d", rc, errno));
+				GRAD_DEBUG(100, ("rc = %d, errno = %d", rc, errno));
 				break;
 			} else if (rc > 0) {
 				rc = write(fd, data, 1);
 				if (rc != 1) {
-					debug(100, ("rc = %d, errno = %d", rc,
+					GRAD_DEBUG(100, ("rc = %d, errno = %d", rc,
                                               errno));
 					break;
 				}
@@ -117,7 +118,7 @@ pipe_write(int fd, void *ptr, size_t size, struct timeval *tv)
 				n++;
 			}
 		}
-		debug(100,("n = %d",n));
+		GRAD_DEBUG(100,("n = %d",n));
 		return n;
 	}
 }
@@ -163,8 +164,10 @@ pipe_read(int fd, void *ptr, size_t size, struct timeval *tv)
 			}
 
 			rc = select(fd + 1, &rd_set, NULL, NULL, &tval);
-			if (rc == 0)
+			if (rc == 0) {
+				errno = ETIMEDOUT;
 				break;
+			}
 			if (rc < 0) {
 				if (errno == EINTR)
 					continue;
@@ -194,7 +197,7 @@ rpp_fd_read(int fd, void *data, size_t size, struct timeval *tv)
 	sz = pipe_read(fd, &nbytes, sizeof(nbytes), tv);
 	if (sz == 0)
 		return 0; /* eof */
-	debug(100,("nbytes=%lu",nbytes));
+	GRAD_DEBUG(100,("nbytes=%lu",nbytes));
 	if (sz != sizeof(nbytes)) 
 		return -1;
 	sz = nbytes > size ? size : nbytes;
@@ -206,7 +209,7 @@ rpp_fd_read(int fd, void *data, size_t size, struct timeval *tv)
 			return -3;
 	}
 	
-	debug(100,("return %lu", (unsigned long)sz));
+	GRAD_DEBUG(100,("return %lu", (unsigned long)sz));
 	return sz;
 }
 
@@ -215,12 +218,12 @@ static int
 rpp_fd_write(int fd, void *data, size_t size, struct timeval *tv)
 {
 	int rc;
-	debug(100,("size=%lu",size));
+	GRAD_DEBUG(100,("size=%lu",size));
 	if (pipe_write(fd, &size, sizeof(size), tv) != sizeof(size)) 
 		return -1;
 	if (pipe_write(fd, data, size, tv) != size) 
 		return -2;
-	debug(1,("return %lu", (unsigned long)size));
+	GRAD_DEBUG(1,("return %lu", (unsigned long)size));
 	return size;
 }
 
@@ -332,17 +335,19 @@ rpp_lookup_ready(int (*proc_main)(void *), void *data)
 }
 
 /* Find the rpp_proc_t with a given PID */
+static int
+rpp_comparator(void *item, void *data)
+{
+	rpp_proc_t *p = item;
+	pid_t *pid = data;
+	return p->pid != *pid;
+}
+
+/* NOTE: Do not use any memory allocation calls */
 rpp_proc_t *
 rpp_lookup_pid(pid_t pid)
 {
-	rpp_proc_t *p;
-	grad_iterator_t *itr = grad_iterator_create(process_list);
-	for (p = grad_iterator_first(itr); p; p = grad_iterator_next(itr)) {
-		if (p->pid == pid)
-			break;
-	}
-	grad_iterator_destroy(&itr);
-	return p;
+	return grad_list_locate(process_list, &pid, rpp_comparator);
 }
 
 /* Remove the given rpp_proc_t from the list. The underlying process
@@ -478,7 +483,7 @@ rpp_kill(pid_t pid, int signo)
 		rpp_proc_t *p = rpp_lookup_pid(pid);
 		if (p) {
 			kill(p->pid, signo);
-			_rpp_remove(p);
+			rpp_check_pid(p->pid);
 		} else
      		        return 1;
 	} else 
@@ -486,6 +491,23 @@ rpp_kill(pid_t pid, int signo)
 	return 0;
 }
 
+pid_t
+rpp_check_pid(pid_t pid)
+{
+	int status;
+	rpp_proc_t *p = rpp_lookup_pid(pid);
+	if (!p)
+		return -1;
+	if (p->status != process_finished) {
+		pid_t npid = waitpid(pid, &status, WNOHANG);
+		if (npid <= 0)
+			return pid;
+		rpp_status_changed(pid, status);
+	}
+	rpp_remove (p);
+	return 0;
+}
+
 /* Slay the child */
 static void
 _rpp_slay(rpp_proc_t *p, char *msg)
@@ -537,7 +559,7 @@ rpp_forward_request(REQUEST *req)
 
 	if (!p)
 		return 1;
-	debug(1, ("sending request to %d", p->pid));
+	GRAD_DEBUG(1, ("sending request to %d", p->pid));
 	
 	frq.type = req->type;
 	frq.addr = req->addr;
@@ -595,6 +617,7 @@ sig_handler(int sig)
 		
 	case SIGTERM:
 	case SIGQUIT:
+		/* FIXME: Possibly unsafe, it calls free() etc. */
 	        radiusd_exit0();
 
 	case SIGCHLD:
@@ -662,7 +685,7 @@ rpp_request_handler(void *arg ARG_UNUSED)
 		rc = request_handle(req, request_respond);
 			
 		/* Inform the master */
-		debug(1, ("notifying the master"));
+		GRAD_DEBUG(1, ("notifying the master"));
 		repl.code = RPP_COMPLETE;
 		repl.size = 0;
 		rpp_fd_write(rpp_stdout, &repl, sizeof repl, NULL);
@@ -705,7 +728,7 @@ rpp_input_handler(int fd, void *data)
 		}
 		
 		if (p) {
-		        debug(1, ("updating pid %d", p->pid));
+		        GRAD_DEBUG(1, ("updating pid %d", p->pid));
 			p->status = process_ready;
 			request_update(p->pid, RS_COMPLETED, data);
 		} 
