@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
  
@@ -221,7 +221,7 @@ static struct argp argp = {
         parse_opt,
         NULL,
         doc,
-        rad_common_argp_child,
+        grad_common_argp_child,
         NULL, NULL
 };
 
@@ -263,7 +263,7 @@ int
 read_naslist()
 {
         int rc;
-        char *path = grad_mkfilename(radius_dir, RADIUS_NASLIST);
+        char *path = grad_mkfilename(grad_config_dir, RADIUS_NASLIST);
         rc = grad_nas_read_file(path);
         grad_free(path);
         return rc;
@@ -355,7 +355,7 @@ radwtmp()
         /*time(&buf[0].ut_time);*/
         
         while (!stop && --bl >= 0) {
-                if (lseek(wfd, (off_t)(bl * sizeof(buf)), L_SET) == -1 ||
+                if (lseek(wfd, (off_t)(bl * sizeof(buf)), SEEK_SET) == -1 ||
                     (bytes = read(wfd, buf, sizeof(buf))) == -1)
                         grad_log(L_ERR, "%s", file);
                 for (bp = &buf[bytes / sizeof(buf[0]) - 1]; !stop && bp >= buf;
